@@ -478,7 +478,7 @@ struct nlattr *nla_reserve(struct nl_msg *n, int attrtype, int attrlen)
 	
 	tlen = NLMSG_ALIGN(n->nm_nlh->nlmsg_len) + nla_total_size(attrlen);
 
-	if ((tlen + n->nm_nlh->nlmsg_len) > n->nm_size) {
+	if (tlen > n->nm_size) {
 		nl_errno(ENOBUFS);
 		return NULL;
 	}
